#! /bin/bash

LibPath=
TmpPath="/tmp/Add-static-lib.a-tmp"
aname="Add-static-lib.a"
alib=
all_a="0"
clr_a="0"
cp_a="0"
USE_RANLIB="0"
RANLIB=ranlib

for option ; do
  case $option in
  *=?*) optarg=`expr "X$option" : '[^=]*=\(.*\)'` ;;
  *=)   optarg= ;;
  *)    optarg= ;;
  esac

  case $option in
  -LibPath=*) LibPath=$optarg ;;
  -TmpPath=*) TmpPath=$optarg ;;
  -aname=*) aname=$optarg ;;
  -alib=*) alib=$optarg ;;
  -all_a) all_a="1" ;;
  -clr_a) clr_a="1" ;;
  -cp_a) cp_a="1" ;;
  -ranlib) USE_RANLIB="1" ;;
  -ranlib=*) USE_RANLIB="1" ; RANLIB=$optarg ;;
  *) echo "ERROR: arg $option" ; exit 1 ;;
  esac

done

if test "$all_a" = "1" ; then
  pushd $LibPath
  alib=`echo *.a`
  popd
fi

if test -z "$alib" ; then
  echo "ERROR: no lib.a data"
  exit 1
fi

mkdir -p $TmpPath
test $? != 0 && exit 1
cd $TmpPath
test $? != 0 && exit 1

rm -f *

add_archive() {
  local objs f fx xN add1objs add2objs add3objs

  echo "add arcive $1"

  add1objs=

  objs=`ar -t $LibPath/$1`
  test $? != 0 && exit 1

  xN="0"
  for f in $objs ; do
    for fx in $add1objs ; do
      if test $fx = $f ; then
	echo "have N2 $f"
	xN="2"
      fi
    done
    add1objs="$add1objs $f"    
  done

  if test "$xN" = "0" ; then
    ar -x $LibPath/$1
    ar qc $aname $objs
    rm *.o
    return
  fi

  add1objs=
  add2objs=
  add3objs=

  for f in $objs ; do

    xN="0"

    for fx in $add3objs ; do
      if test $fx = $f ; then
	echo "add N4 $f"
	xN="4"
      fi
    done

    if test $xN = "0" ; then
      for fx in $add2objs ; do
	if test $fx = $f ; then
	  echo "add N3 $f"
	  xN="3"
	fi
      done
    fi

    if test $xN = "0" ; then
      for fx in $add1objs ; do
	if test $fx = $f ; then
	  echo "add N2 $f"
	  xN="2"
	fi
      done
    fi

    case "$xN" in
    0)
      add1objs="$add1objs $f"
      ar -x $LibPath/$1 $f
      test $? != 0 && exit 1
      ;;
    2)
      add2objs="$add2objs $f"
      ar -xN $xN $LibPath/$1 $f
      test $? != 0 && exit 1
      ;;
    3)
      add3objs="$add3objs $f"
      ar -xN $xN $LibPath/$1 $f
      test $? != 0 && exit 1
      ;;
    *)
      echo "ERROR: in archive more object modules $xN $f"
      exit 1
      ;;
    esac
    ar qc $aname $f
    test $? != 0 && exit 1
    rm $f
  done
}

for f in $alib ; do
  add_archive $f
  if test "$clr_a" = "1" ; then rm $LibPath/$f ; fi
done

if test "$USE_RANLIB" = "1" ; then
  $RANLIB $TmpPath/$aname
  test $? != 0 && exit 1
fi

if test "$cp_a" = "1" ; then
  mv $TmpPath/$aname $LibPath/$aname
  test $? != 0 && exit 1
  rm -rf $TmpPath
  test $? != 0 && exit 1
fi

exit 0
